{*******************************************************}
{                                                       }
{        THintEngine  - non-visual component for        }
{                   hints developer :)                  }
{    Visualisation some events - Application.OnHint,    }
{    Application.OnShowHint. Additional events:         }
{       OnHideHint, OnDrawHint, OnActivateHint.         }
{                                                       }
{      Copyright (c) 1997 by Ilya Andreev AKA Andre     }
{                                                       }
{                     SPb, Russia.                      }
{                                                       }
{                       Freeware.                       }
{                                                       }
{       FIDONet: 2:5030/55.28 AKA 2:5030/402.17         }
{                                                       }
{           Please, send me bag reports :)              }
{                                                       }
{*******************************************************}

unit HintEng;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  DsgnIntf;
var
  ApplicationHintEngine : longint;

type
  THintStyle = (hsStandart, hsTransparent, hsOwnerDraw);
  TAbout = (About);

  TDrawEvent = procedure(HintWindow : THintWindow;
          var NeedDraw : boolean) of object;
  THintActivateEvent = procedure(var Height, Width : integer; Rect: TRect) of object;
  THintHideEvent = procedure(HintWindow : THintWindow;
          var CanHide : boolean) of object;

  TAboutProperty = class(TPropertyEditor)
  public
    procedure Edit; override;
    function GetAttributes: TPropertyAttributes; override;
    function GetValue: string; override;
  end;

  THintEngine = class(TComponent)
  private
    FAbout : TAbout;
    FOnShowHint : TShowHintEvent;
    FOnHint : TNotifyEvent;
    FOldOnShowHint : TShowHintEvent;
    FOldOnHint : TNotifyEvent;
    FActivateHint : THintActivateEvent;
    FHintColor : TColor;
    FHintFont : TFont;
    FHintPause : integer;
    FHintHidePause : integer;
    FHintShortPause : integer;
    FShowHint : boolean;
    FStyle : THintStyle;
    FOnDraw : TDrawEvent;
    FOnHintHide : THintHideEvent;
    FHintWindow : THintWindow;
    FHintControl : TControl;
    procedure SetFont(Value : TFont);
    procedure SetColor(Value : TColor);
    procedure SetPause(Value : integer);
    procedure SetHidePause(Value : integer);
    procedure SetShortPause(Value : integer);
    { Private declarations }
  protected
    procedure EngineOnHint(Sender: TObject);
    procedure EngineOnShowHint(var HintStr: string;
        var CanShow: Boolean; var HintInfo: THintInfo);
    procedure Loaded; override;
    { Protected declarations }
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    property HintWindow : THintWindow read FHintWindow;
    property HintControl : TControl read FHintControl;
    { Public declarations }
  published
    property About : TAbout read FAbout write FAbout;
    property HintColor : TColor read FHintColor write SetColor;
    property HintPause : integer read FHintPause write SetPause;
    property HintHidePause : integer read FHintHidePause write SetHidePause;
    property HintShortPause : integer read FHintShortPause write SetShortPause;
    property HintFont : TFont read FHintFont write SetFont;
    property ShowHint : boolean read FShowHint write FShowHint;
    property Style : THintStyle read FStyle write FStyle;
    property OnShowHint : TShowHintEvent read FOnShowHint write FOnShowHint;
    property OnHint : TNotifyEvent read FOnHint write FOnHint;
    property OnDraw : TDrawEvent read FOnDraw write FOnDraw;
    property OnActivateHint : THintActivateEvent read FActivateHint write FActivateHint;
    property OnHintHide : THintHideEvent read FOnHintHide write FOnHintHide;
    { Published declarations }
  end;

  THintEnWindow = class(THintWindow)
   private
    FHintEngine : THintEngine;
    procedure CreateParams(var Params: TCreateParams);override;
    procedure ActivateHint(Rect: TRect; const AHint: string); override;
    function  isHintMsg(var Msg: TMsg): Boolean;override;
    procedure WMEraseBkgnd(var Message: TWMEraseBkgnd); message WM_ERASEBKGND;
    procedure WMGetDLGCode(var Message: TMessage); message WM_GETDLGCODE;
   public
    property Canvas;
    procedure Paint; override;
    constructor Create(AOwner: TComponent); override;
  end;

procedure Register;

implementation

procedure THintEnWindow.CreateParams(var Params: TCreateParams);
begin
 inherited CreateParams(Params);
 if (FHintEngine <> nil) and (FHintEngine.Style = hsTransparent)
   then Params.ExStyle := Params.ExStyle or ws_ex_Transparent;
 if (FHintEngine <> nil) and (FHintEngine.Style <> hsStandart)
   then Params.Style := Params.Style and not WS_BORDER;
end;

procedure THintEnWindow.WMGetDLGCode(var Message: TMessage);
begin
  Message.Result := DLGC_WANTARROWS;
end;

procedure THintEnWindow.WMEraseBkgnd(var Message: TWMEraseBkgnd);
begin
  Message.Result := 1;
end;

constructor THintEnWindow.Create(AOwner: TComponent);
begin
 inherited Create(AOwner);
 FHintEngine := THintEngine(ApplicationHintEngine);
 if FHintEngine <> nil then
   begin
     Canvas.Font := FHintEngine.HintFont;
     if (FHintEngine.Style = hsStandart)
       then Canvas.Brush.Color := FHintEngine.HintColor;
     FHintEngine.FHintWindow := Self;
    end;
end;

procedure THintEnWindow.ActivateHint(Rect: TRect; const AHint: string);
var h,w : integer;
begin
  Caption := AHint;
  BoundsRect := Rect;
  H := Height; W := Width;
  if Rect.Top + Height > Screen.Height then
    Rect.Top := Screen.Height - Height;
  if Rect.Left + Width > Screen.Width then
    Rect.Left := Screen.Width - Width;
  if Rect.Left < 0 then Rect.Left := 0;
  if Rect.Bottom < 0 then Rect.Bottom := 0;

  if (FHintEngine <> nil) and Assigned(FHintEngine.OnActivateHint)
    then FHintEngine.OnActivateHint(H, W, Rect);

  SetWindowPos(Handle, HWND_TOPMOST, Rect.Left, Rect.Top, W,
    H, SWP_SHOWWINDOW or SWP_NOACTIVATE {or SWP_FRAMECHANGED  {or SWP_NOSIZE});
end;

procedure THintEnWindow.Paint;
var NeedDraw : boolean;
    R        : TRect;
begin
 NeedDraw := true;
 if (FHintEngine <> nil) and Assigned(FHintEngine.OnDraw)
   then FHintEngine.OnDraw(Self, NeedDraw);
 if NeedDraw then begin
   R := ClientRect;
   if (FHintEngine <> nil) and (FHintEngine.Style = hsStandart)
     then Canvas.FillRect(Rect(0,0,Width,Height));
   Inc(R.Left, 1);
   DrawText(Canvas.Handle, PChar(Caption), -1, R, DT_LEFT or DT_NOPREFIX or
   DT_WORDBREAK+ DT_NoClip);
  end;
end;

function THintEnWindow.IsHintMsg(var Msg: TMsg): Boolean;
begin
 result := inherited IsHintMsg(Msg);
 if (FHintEngine <> nil) and
  Assigned(FHintEngine.OnHintHide) then FHintEngine.OnHintHide(Self, result);
 if (FHintEngine <> nil) and result
   then FHintEngine.FHintControl := nil;
end;

constructor THintEngine.Create(AOwner: TComponent);
begin
 inherited Create(AOwner);
 FHintFont := TFont.Create;
 HintColor := clYellow;
 HintPause := 800;
 HintHidePause := 2500;
 HintShortPause := 50;
 ShowHint := true;
 if not (csDesigning in ComponentState) then
   with Application do begin
     FOldOnShowHint := OnShowHint;
     FOldOnHint := OnHint;
     OnShowHint := EngineOnShowHint;
     OnHint := EngineOnHint;
     HintWindowClass := THintEnWindow;
     ApplicationHintEngine := Longint(Self);
    end;
end;

destructor THintEngine.Destroy;
begin
 Application.OnShowHint := FOLdOnShowHint;
 Application.OnHint := FOLdOnHint;
 FHintFont.Free;
 inherited Destroy;
end;

procedure THintEngine.EngineOnHint(Sender: TObject);
begin
 if Assigned(FOnHint) then FOnHint(Sender);
end;

procedure THintEngine.EngineOnShowHint(var HintStr: string;
     var CanShow: Boolean; var HintInfo: THintInfo);
begin
 FHintControl := HintInfo.HintControl;
 if Assigned(FOnShowHint) then FOnShowHint(HintStr, CanShow, HintInfo);
end;

procedure THintEngine.SetColor(Value : TColor);
begin
 FHintColor := Value;
 Application.HintColor := Value;
end;

procedure THintEngine.SetPause(Value : integer);
begin
 FHintPause := Value;
 Application.HintPause := Value;
end;

procedure THintEngine.SetHidePause(Value : integer);
begin
 FHintHidePause := Value;
 Application.HintHidePause := Value;
end;

procedure THintEngine.SetShortPause(Value : integer);
begin
 FHintShortPause := Value;
 Application.HintShortPause := Value;
end;

procedure THintEngine.Loaded;
begin
 inherited Loaded;
 if not (csDesigning in ComponentState) then
   with Application do begin
     ShowHint := false;
     ShowHint := FShowHint;
    end;
end;

procedure THintEngine.SetFont(Value : TFont);
begin
 FHintFont.Assign(Value);
 if FHintWindow <> nil then FHintWindow.Canvas.Font.Assign(Value);
end;

// Cool Editor :)))

procedure TAboutProperty.Edit;
begin
 MessageBox(Application.Handle,
   PChar('Power HintEngine version 1.01' + #13 +
         'Copyright by Andre. Shareware: 5$'+ #13 +
         'SPb, Russia. Fidonet: 2:5030/55.28' + #13 +
         'email: andre@bz-tech.spb.su'),
          'About HintEngine', MB_ICONASTERISK + mb_OK);
end;

function TAboutProperty.GetAttributes: TPropertyAttributes;
begin
  Result := [paDialog, paReadOnly];
end;

function TAboutProperty.GetValue: string;
begin
  Result := 'Copyright by Andre';
end;

procedure Register;
begin
  RegisterComponents('Andre VCL', [THintEngine]);
  RegisterPropertyEditor(TypeInfo(TAbout),nil,'About', TAboutProperty);
end;



end.
